{                                                                          }
{    Specific imports from the GPC Dos unit for the Sysutils unit          }
{                                                                          }
{    Copyright (C) 1997-2001 Free Software Foundation, Inc.                }
{                                                                          }
{    Authors:                                                              }
{       Frank Heckenbach <frank@g-n-u.de>                                  }
{       Prof Abimbola A Olowofoyeku <African_Chief@bigfoot.com>            }
{                                                                          }
{    This file is part of GNU Pascal.                                      }
{
     Routines imported from the Dos unit
         FindFirst,
         FindNext,
         FindClose
}
{                                                                          }
{                                                                          }
{    This library is a free library; you can redistribute and/or modify    }
{    it under the terms of the GNU Library General Public License,         }
{    version 2, as published by the Free Software Foundation.              }
{                                                                          }
{    This library is distributed in the hope that it will be useful,       }
{    but WITHOUT ANY WARRANTY; without even the implied warranty of        }
{    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         }
{    GNU Library General Public License for more details.                  }
{                                                                          }
{    You should have received a copy of the GNU Library General Public     }
{    License along with this library (see the file COPYING.LIB);           }
{    if not, write to the Free Software Foundation, Inc., 675 Mass Ave,    }
{    Cambridge, MA 02139, USA.                                             }
{                                                                          }
{    As a special exception, if you link this file with files compiled     }
{    with a GNU compiler to produce an executable, this does not cause     }
{    the resulting executable to be covered by the GNU Library General     }
{    Public License. This exception does not however invalidate any other  }
{    reasons why the executable file might be covered by the GNU Library   }
{    General Public License.                                               }
{                                                                          }

{ Since there's no explicit closing of FindFirst/FindNext, FindList keeps
  tracks of all running searches so they can be closed automatically when
  necessary, and Magic indicates if a SearchRec is currently in use. }

CONST
  srOpened = $2424d00f;
  srDone   = $4242f00d;

TYPE
  TSRFillInternal = PACKED RECORD
    Magic  : Integer;
    Unused : PACKED ARRAY [1..SizeOf ( SearchRecFill ) - SizeOf ( Integer ) ] OF Byte
  END;

  PPFindList = ^PFindList;
  PFindList  = ^TFindList;
  TFindList  = RECORD
    Next : PFindList;
    SR   : ^SearchRec;
    Dir,
    Name,
    Ext  : TString;
    Attr : TDosAttr;
    PDir : Pointer;
  END;

VAR
  FindList : PFindList = nil;

PROCEDURE CloseFind ( PTemp : PPFindList );
VAR Temp : PFindList;
BEGIN
  Temp := PTemp^;
  CloseDir ( Temp^.PDir );
  TSRFillInternal ( Temp^.SR^.Fill ) .Magic := srDone;
  PTemp^ := Temp^.Next;
  Dispose ( Temp )
END;


FUNCTION FindFirst;
{ (const Path : String; Attr : TDosAttr; var SR : TSearchRec);}
VAR
  Temp : PFindList;
  PTemp : PPFindList;
BEGIN
  { If SR.DosData was used before, close it first }
  PTemp := @FindList;
  WHILE ( PTemp^ <> nil ) AND ( PTemp^^.SR <> @SR.DosData ) DO PTemp := @PTemp^^.Next;
  IF PTemp^ <> nil THEN
    BEGIN
      CloseFind ( PTemp );
      IF IOResult <> 0 THEN DosError := DosError_ReadFault
    END;

  IF fAttr AND not ( ReadOnly OR Archive ) = VolumeID THEN
    BEGIN
      DosError := DosError_NoMoreFiles;
      Result := DosError;
      Exit
    END;

  New ( Temp );
  FSplit ( Path, Temp^.Dir, Temp^.Name, Temp^.Ext );
  IF Temp^.Dir = '' THEN Temp^.Dir := DirSelf + DirSeparator;
  IF Temp^.Ext = '' THEN Temp^.Ext := ExtSeparator;
  Temp^.SR := @SR.DosData;
  Temp^.Attr := fAttr;
  Temp^.PDir := OpenDir ( RemoveDirSeparator ( Temp^.Dir ) );
  IF IOResult <> 0
  THEN BEGIN
      TSRFillInternal ( SR.DosData.Fill ) .Magic := srDone;
      Dispose ( Temp );
      DosError := DosError_NoMoreFiles;
      Result := DosError;
      Exit
    END;
  TSRFillInternal ( SR.DosData.Fill ) .Magic := srOpened;
  Temp^.Next := FindList;
  FindList := Temp;
  Result := FindNext ( SR );
END;

FUNCTION  FindNext;
{ (var SR : TSearchRec);}
VAR
  Temp : PFindList;
  PTemp : PPFindList;
  Name, Dir, Nam, Ext : TString;
  F : Text;
  TmpAttr : TDosAttr;
  TmpSize,
  TmpTime : Longint;

  { Emulate Dos brain-damaged file name wildcard matching }
  FUNCTION MatchPart ( CONST aName, Mask : String ) : Boolean;
  VAR i : Integer;
  BEGIN
    FOR i := 1 TO Length ( Mask ) DO
      CASE Mask [i] OF
        '?' : ;
        '*' : return True;
        ELSE
          IF ( i > Length ( aName ) ) OR ( FileNameLoCase ( aName [i] ) <> FileNameLoCase ( Mask [i] ) ) THEN return False
      END;
    MatchPart := Length ( Mask ) >= Length ( aName )
  END;

BEGIN
  DosError := 0;
  TmpAttr := 0; {}
  Result := DosError;
  { Check if SR.DosData is still valid }
  CASE TSRFillInternal ( SR.DosData.Fill ) .Magic OF
    srOpened : ;
    srDone   : BEGIN
                 DosError := DosError_NoMoreFiles;
                 Result := DosError;
                 Exit
               END;
    ELSE
      DosError := DosError_InvalidMem;
      Result := DosError;
      Exit
  END;
  PTemp := @FindList;
  WHILE ( PTemp^ <> nil ) AND ( PTemp^^.SR <> @SR.DosData ) DO PTemp := @PTemp^^.Next;
  Temp := PTemp^;
  IF Temp = nil THEN
    BEGIN
      DosError := DosError_InvalidMem;
      Result := DosError;
      Exit
    END;
  REPEAT
    Name := ReadDir ( Temp^.PDir );
    IF Name = '' THEN
      BEGIN
        CloseFind ( PTemp );
        IF IOResult = 0
          THEN DosError := DosError_NoMoreFiles
          ELSE DosError := DosError_ReadFault;
        Result := DosError;
        Exit
      END;

    Assign ( F, Temp^.Dir + Name );
    GetFAttr ( F, TmpAttr );

    SR.DosData.Attr := TmpAttr;
    SR.Attr := TmpAttr; { @@ }
    FSplit ( Name, Dir, Nam, Ext );
    IF Ext = '' THEN Ext := ExtSeparator;
  UNTIL MatchPart ( Nam, Temp^.Name ) AND MatchPart ( Ext, Temp^.Ext ) AND
        { Emulate Dos brain-damaged file attribute matching }
        ( ( Temp^.Attr AND ( Hidden OR SysFile ) <> 0 ) OR ( TmpAttr AND Hidden    = 0 ) ) AND
        ( ( Temp^.Attr AND Directory             <> 0 ) OR ( TmpAttr AND Directory = 0 ) );
  SR.DosData.Name := Name;
  Result := DosError;
  IF DosError <> 0 THEN Exit;

  GetFTime ( F, TmpTime );
  GetFAttr ( F, TmpAttr );  { @ }
  SR.DosData.Time := TmpTime;

  IF Binding ( F ) .Existing THEN
    BEGIN
      Reset ( F );
      TmpSize := FileSize ( F );
      Close ( F );
      If IOResult <> 0 then begin end;
    END
  ELSE
    TmpSize := 0;
  SR.DosData.Size := TmpSize;

  {  @@ additional @@ }
  SR.Name := Name;
  SR.Time := TmpTime;
  SR.Size := TmpSize;
  SR.Attr := TmpAttr;
  SR.DosData.Attr := SR.Attr;

  WITH SR.FindData
  DO BEGIN
     dwFileAttributes := SR.Attr;
     nFileSizeHigh    := SR.Size;
     nFileSizeLow     := SR.Size;
     StrPCopy ( CFILENAME, SR.Name );
     StrLCopy ( CALTERNATEFILENAME, CFILENAME, 14 );
     WITH ftLastWriteTime DO BEGIN
          dwLowDateTime := SR.Size;
          dwHighDateTime := 0;  { @@ crude @@ }
     END;
     ftCreationTime := ftLastWriteTime;
     ftLastAccessTime := ftLastWriteTime;
  END;
  { @@ end of additional @@ }
END;

PROCEDURE FindClose{ ( VAR SR : TSearchRec )};
VAR PTemp : PPFindList;
BEGIN
  PTemp := @FindList;
  WHILE ( PTemp^ <> nil ) AND ( PTemp^^.SR <> @SR.DosData ) DO PTemp := @PTemp^^.Next;
  IF PTemp^ <> nil THEN
    BEGIN
      CloseFind ( PTemp );
      IF IOResult <> 0 THEN DosError := DosError_ReadFault
    END
END;

